//
//  BGM800View.swift
//  MicroLifeDeviceSDK-Demo_Swift
//
//  Created by 吳憲有 on 2024/5/31.
//

import SwiftUI

struct BGM800View: View {
    @Environment(AppInfo.self) var appInfo

    @Environment(\.horizontalSizeClass) var horizontalSizeClass

    @Environment(\.verticalSizeClass) var verticalSizeClass

    var viewModel = BGM800ViewModel()

    var menuInfo: MenuInfo

    var body: some View {
        let orientation = DeviceOrientation.checkOrientation(horizontalSizeClass: horizontalSizeClass, verticalSizeClass: verticalSizeClass)
        ZStack {
            switch orientation {
            case .iPhoneHorizontal, .OtherDevice:
                HorizontalLayoutView(scale: 0.85) {
                    viewModel.startScan()
                } cancelScanAction: {
                    viewModel.cancelScan()
                } disconnectDeviceAction: {
                    viewModel.disconnectDevice()
                } commandViewContent: {
                    CustomBGM800View(viewModel: viewModel)
                }
            case .iPhoneVertical, .iPad:
                VerticalLayoutView(scale: 0.75) {
                    viewModel.startScan()
                } cancelScanAction: {
                    viewModel.cancelScan()
                } disconnectDeviceAction: {
                    viewModel.disconnectDevice()
                } commandViewContent: {
                    CustomBGM800View(viewModel: viewModel)
                }
            }
        }
        .toolbar {
            CustomToolbarItem(
                filePath: log.filePath,
                fileName: log.fileName,
                refreshAction: {
                    viewModel.refreshData()
                })
        }
        .navigationTitle(menuInfo.title)
        .padding(.horizontal, 8)
        .onChange(of: appInfo.showViewType) {
            if appInfo.showViewType == .bgm800 {
                viewModel.setupBLESDK()
            }
        }
        .onAppear {
            appInfo.showViewType = .bgm800
            viewModel.setupBLESDK()
        }
        .onDisappear {
            appInfo.showViewType = .menu
            viewModel.disconnectDevice()
        }
    }
}

struct CustomBGM800View: View {
    var viewModel: BGM800ViewModel

    @State private var selectedRecordCount = ""

    @State private var BTModuleName = ""

    var body: some View {
        VStack(alignment: .trailing, spacing: 16) {
            if viewModel.recordCounts > 0 {
                HStack {
                    Text("Record Count:")
                    Picker("", selection: Binding(
                        get: {
                            viewModel.selectedRecordCount
                        },
                        set: { newValue in
                            viewModel.selectedRecordCount = newValue
                        }
                    )) {
                        ForEach(0 ..< viewModel.recordCounts, id: \.self) { val in
                            Text("\(val)").tag(val)
                        }
                    }
                }
            }
            TextField("BTModuleName", text: $BTModuleName, onCommit: {
                viewModel.BTModuleName = BTModuleName
                // Dismiss the keyboard
                UIApplication.shared.sendAction(#selector(UIResponder.resignFirstResponder), to: nil, from: nil, for: nil)
            })
            .textFieldStyle(RoundedBorderTextFieldStyle())
            .keyboardType(.default)
            Button(action: {
                viewModel.readDeviceName()
            }) {
                Text("Read Device Name")
            }
            Button(action: {
                viewModel.readFWVersion()
            }) {
                Text("Read FW Version")
            }
            Button(action: {
                viewModel.getBLEStatus()
            }) {
                Text("Get BLE Status")
            }
            Button(action: {
                viewModel.readLastData()
            }) {
                Text("Read Last Data")
            }
            Button(action: {
                viewModel.readData()
            }) {
                Text("Read Data")
            }
            Button(action: {
                viewModel.readDataLength()
            }) {
                Text("Read Data Length")
            }
            Button(action: {
                viewModel.uploadMeterData()
            }) {
                Text("Upload Meter Data")
            }
            Button(action: {
                viewModel.disconnect()
            }) {
                Text("Disconnect")
            }
            Button(action: {
                viewModel.readDateTime()
            }) {
                Text("Read DateTime")
            }
            Button(action: {
                viewModel.writeDeviceName()
            }) {
                Text("Write Device Name")
            }
            Button(action: {
                viewModel.getMeterDataTransmissionStatus()
            }) {
                Text("Get Meter Data Transmission Status")
            }
            Button(action: {
                viewModel.getCommunicationMode()
            }) {
                Text("Get Communication Mode")
            }
            Button(action: {
                viewModel.setBLEParingMode()
            }) {
                Text("Set BLE Paring Mode")
            }
            Button(action: {
                viewModel.setMeterAuthorization()
            }) {
                Text("Set Meter Authorization")
            }
            Button(action: {
                viewModel.setParingRequest()
            }) {
                Text("Set Paring Request")
            }
            Button(action: {
                viewModel.setParingTimeOut()
            }) {
                Text("Set Paring TimeOut")
            }
        }
        .padding()
    }
}

#Preview {
    BGM800View(menuInfo: MenuInfo())
}
