//
//  ContentView.swift
//  MicroLifeDeviceSDK-Demo_Swift
//
//  Created by 吳憲有 on 2024/1/18.
//

import SwiftUI
import TipKit

struct ContentView: View {
    var body: some View {
        VStack {
            Image(systemName: "globe")
                .imageScale(.large)
                .foregroundStyle(.tint)
            Text("Hello, world!")
        }
        .padding()
    }
}

/// 通用畫面
struct CustomToolbarItem: ToolbarContent {
    var filePath: String
    var fileName: String
    let refreshAction: () -> Void
    var body: some ToolbarContent {
        ToolbarItem(placement: .navigationBarTrailing) {
            HStack {
                Button(action: {
                    refreshAction()
                }) {
                    Image(systemName: "arrow.clockwise")
                }
                Spacer()
                ShareLink(
                    item: LogText(fileURL: URL(fileURLWithPath: filePath)),
                    preview: SharePreview(fileName)
                )
            }
        }
    }
}

struct CustomDeviceView: View {
    var alignment: HorizontalAlignment = .leading

    var startSacnAction: () -> Void
    var cancelScanAction: () -> Void
    var disconnectDeviceAction: () -> Void

    var body: some View {
        VStack(alignment: alignment, spacing: 16) {
            Button(action: startSacnAction) {
                Text("Start Scan")
            }

            Button(action: cancelScanAction) {
                Text("Cancel Scan")
            }

            Button(action: disconnectDeviceAction) {
                Text("Disconnect Device")
            }
        }
        .padding()
    }
}

struct VerticalLayoutView<Content: View>: View {
    var scale: CGFloat

    var alignment: HorizontalAlignment = .leading

    var startSacnAction: () -> Void
    var cancelScanAction: () -> Void
    var disconnectDeviceAction: () -> Void

    var commandViewContent: () -> Content

    var body: some View {
        ZStack {
            Color.currentBackground(alpha: 0.000001)
                .onTapGesture {
                    // Dismiss the keyboard
                    UIApplication.shared.sendAction(#selector(UIResponder.resignFirstResponder), to: nil, from: nil, for: nil)
                }
            VStack {
                LogView()
                    .containerRelativeFrame(.vertical) { height, _ in height * scale }
                HStack {
                    VStack {
                        CustomDeviceView {
                            startSacnAction()
                        } cancelScanAction: {
                            cancelScanAction()
                        } disconnectDeviceAction: {
                            disconnectDeviceAction()
                        }
                        Spacer()
                    }
                    ScrollView {
                        commandViewContent()
                    }
                    .keyboardAwarePadding() // Apply the keyboard aware padding
                }
                .frame(maxWidth: .infinity, maxHeight: .infinity, alignment: .bottom)
            }
        }
    }
}

struct HorizontalLayoutView<Content: View>: View {
    var scale: CGFloat

    var alignment: HorizontalAlignment = .leading

    var startSacnAction: () -> Void
    var cancelScanAction: () -> Void
    var disconnectDeviceAction: () -> Void

    var commandViewContent: () -> Content

    var body: some View {
        ZStack {
            Color.currentBackground(alpha: 0.000001)
                .onTapGesture {
                    // Dismiss the keyboard
                    UIApplication.shared.sendAction(#selector(UIResponder.resignFirstResponder), to: nil, from: nil, for: nil)
                }
            HStack {
                LogView()
                    .containerRelativeFrame(.horizontal) { width, _ in width * scale }
                ScrollView {
                    CustomDeviceView {
                        startSacnAction()
                    } cancelScanAction: {
                        cancelScanAction()
                    } disconnectDeviceAction: {
                        disconnectDeviceAction()
                    }
                    Spacer()
                    commandViewContent()
                }
                .keyboardAwarePadding() // Apply the keyboard aware padding
            }
        }
    }
}

enum MeasurementTime: Int, CaseIterable, Identifiable {
    case off = 0
    case five = 5
    case ten = 10
    case fifteen = 15
    case twenty = 20
    case thirty = 30
    case sixty = 60

    var id: Int { rawValue }
    var description: String {
        switch self {
        case .off: return "Off"
        case .five: return "5"
        case .ten: return "10"
        case .fifteen: return "15"
        case .twenty: return "20"
        case .thirty: return "30"
        case .sixty: return "60"
        }
    }
}

enum HIinfPressure: Int, CaseIterable, Identifiable {
    case auto = 0
    case oneSixty = 160
    case oneEighty = 180
    case twoHundred = 200
    case twoTwenty = 220
    case twoForty = 240
    case twoEighty = 280

    var id: Int { rawValue }
    var description: String {
        switch self {
        case .auto: return "Auto"
        case .oneSixty: return "160"
        case .oneEighty: return "180"
        case .twoHundred: return "200"
        case .twoTwenty: return "220"
        case .twoForty: return "240"
        case .twoEighty: return "280"
        }
    }
}

enum Interval: Int, CaseIterable, Identifiable {
    case five = 5
    case ten = 10
    case fifteen = 15
    case twenty = 20
    case thirty = 30
    case sixty = 60
    case off = 255

    var id: Int { rawValue }
    var description: String {
        switch self {
        case .five: return "5"
        case .ten: return "10"
        case .fifteen: return "15"
        case .twenty: return "20"
        case .thirty: return "30"
        case .sixty: return "60"
        case .off: return "Off"
        }
    }
}

enum Dformat: Int, CaseIterable, Identifiable {
    case noCBPRaw = 0
    case lowCBPRaw = 1
    case fullCBPRaw = 3

    var id: Int { rawValue }
    var description: String {
        switch self {
        case .noCBPRaw: return "No CBP Raw"
        case .lowCBPRaw: return "Low CBP Raw"
        case .fullCBPRaw: return "Full CBP Raw"
        }
    }
}

enum PARAM: Int, CaseIterable, Identifiable {
    case none = 0x0000
    case all = 0x1100
    case lessSN1 = 0x1200
    case moreSN1 = 0x1300
    case includeSN12 = 0x1400

    var id: Int { rawValue }
    var description: String {
        switch self {
        case .none: return "None"
        case .all: return "All"
        case .lessSN1: return "Less SN1"
        case .moreSN1: return "More SN1"
        case .includeSN12: return "Include SN12"
        }
    }
}

struct CBPindexPicker: View {
    @State var historyMeasuremeCount = 0

    @Binding var CBPindex: Int

    var action: () -> Void

    var body: some View {
        Picker("", selection: Binding(
            get: {
                CBPindex
            },
            set: { newValue in
                CBPindex = newValue
                action()
            }
        )) {
            ForEach(0 ..< historyMeasuremeCount, id: \.self) { index in
                Text("\(index)").tag(index)
            }
        }
    }
}

struct DformatPicker: View {
    @Binding var dformat: Dformat

    var action: () -> Void

    var body: some View {
        Picker("", selection: Binding(
            get: {
                dformat
            },
            set: { newValue in
                dformat = newValue
                action()
            }
        )) {
            ForEach(Dformat.allCases) { format in
                Text(format.description).tag(format)
            }
        }
    }
}

struct PARAMPicker: View {
    @Binding var param: PARAM

    var action: () -> Void

    var body: some View {
        Picker("", selection: Binding(
            get: {
                param
            },
            set: { newValue in
                param = newValue
                action()
            }
        )) {
            ForEach(PARAM.allCases) { param in
                Text(param.description).tag(param)
            }
        }
    }
}

struct SettingValueView: View {
    var showLists: [String] = ["SW_checkhide", "SW_SEL_silent", "CBP_zone1_meas_off", "CBP_zone2_meas_off", "HI_infPressure", "ABPMStart", "ABPMEnd", "ABPMInt_first", "ABPMInt_second", "CBPInt_first", "CBPInt_second"]

    @Binding var settings: [String: Any]

    @Binding var showText: String

    var buttonAction: () -> Void

    var body: some View {
        ScrollView {
            ForEach(settings.keys.sorted(), id: \.self) { key in
                VStack(alignment: .leading) {
                    if showLists.contains(key) {
                        if key == "settingValues" {
                            SettingValueListView(key: key, settings: $settings)
                        } else {
                            HStack {
                                Text(key)
                                Spacer()
                                SettingValueListView(key: key, settings: $settings)
                            }
                            .background(Color.RainbowColor(type: backgroundColorType, alpha: 0.5))
                            .padding(.vertical, 4)
                        }
                    }
                }
            }
            Spacer()
            Button(action: {
                buttonAction()
            }) {
                Text(showText)
            }
        }
        .padding(.all, 16)
    }
}

struct SettingValueListView: View {
    var key: String

    var showItems: [String] = ["No.", "SW_SEL_silent", "BP_meas_off", "CBP_meas_off", "ABPMStart", "ABPMEnd", "Interval"]

    @Binding var settings: [String: Any]

    var body: some View {
        switch key {
        case "ABPMStart", "ABPMEnd":
            if let intValue = settings[key] as? Int {
                Picker("", selection: Binding(
                    get: {
                        intValue
                    },
                    set: { newValue in
                        settings[key] = newValue
                    }
                )) {
                    ForEach(0 ..< 24, id: \.self) { hour in
                        Text("\(hour)").tag(hour)
                    }
                }
            }

        case "ABPMInt_first", "ABPMInt_second", "CBPInt_first", "CBPInt_second":
            if let measurementTimeValue = settings[key] as? Int {
                Picker("", selection: Binding(
                    get: {
                        measurementTimeValue
                    },
                    set: { newValue in
                        settings[key] = newValue
                    }
                )) {
                    ForEach(MeasurementTime.allCases) { time in
                        Text(time.description).tag(time.rawValue)
                    }
                }
            }

        case "CBP_zone2_meas_off", "CBP_zone1_meas_off", "SW_SEL_silent", "SW_checkhide", "BP_meas_off", "CBP_meas_off", "SW_AUTO_hide", "SW_AUS_Hide", "SW_AVG_no_include_first", "SW_CBP", "SW_AFib", "SW_AMPM", "SW_Kpa", "AutoMeasureNumber":
            if let boolValue = settings[key] as? Bool {
                Toggle("", isOn: Binding(
                    get: {
                        boolValue
                    },
                    set: { newValue in
                        settings[key] = newValue
                    }
                ))
            }

        case "AUS_HI_infPressure", "HI_infPressure":
            if let hiInfPressureValue = settings[key] as? Int {
                Picker("", selection: Binding(
                    get: {
                        hiInfPressureValue
                    },
                    set: { newValue in
                        settings[key] = newValue
                    }
                )) {
                    ForEach(HIinfPressure.allCases) { pressure in
                        Text(pressure.description).tag(pressure.rawValue)
                    }
                }
            }

        case "Interval":
            if let interval = settings[key] as? Int {
                Picker("", selection: Binding(
                    get: {
                        interval
                    },
                    set: { newValue in
                        settings[key] = newValue
                    }
                )) {
                    ForEach(Interval.allCases) { pressure in
                        Text(pressure.description).tag(pressure.rawValue)
                    }
                }
            }

        case "RestTime", "IntervalTime":
            if let intValue = settings[key] as? Int {
                Picker("", selection: Binding(
                    get: {
                        intValue
                    },
                    set: { newValue in
                        settings[key] = newValue
                    }
                )) {
                    ForEach(0 ..< 255, id: \.self) { val in
                        Text("\(val)").tag(val)
                    }
                }
            }

        case "settingValues":
            if let settingValues = settings[key] as? [[String: Any]] {
                /// 依據"No."排序
                ForEach(settingValues.indices, id: \.self) { index in
                    let sv = settingValues[index]
                    VStack(alignment: .leading) {
                        ForEach(sv.keys.sorted(by: { a, b -> Bool in
                            if a == "No." {
                                return true
                            }
                            if b == "No." {
                                return false
                            }
                            return a < b
                        }), id: \.self) { subKey in
                            HStack {
                                /// showItems 依據做顯示
                                if showItems.contains(subKey) {
                                    Text(subKey)
                                    Spacer()
                                    SettingValueItemView(index: index, key: subKey, settings: $settings)
                                }
                            }
                            .padding(.vertical, 1)
                        }
                    }
                    .background(Color.RainbowColor(type: backgroundColorType, alpha: 0.5))
                }
            }

        default:
            EmptyView()
        }
    }
}

struct SettingValueItemView: View {
    var index: Int

    var key: String

    @Binding var settings: [String: Any]

    var body: some View {
        let settingValues = settings["settingValues"] as! [[String: Any]]

        var settingValue = settingValues[index]

        switch key {
        case "ABPMStart", "ABPMEnd":
            if let intValue = settingValue[key] as? Int {
                Picker("", selection: Binding(
                    get: {
                        intValue
                    },
                    set: { newValue in
                        settingValue[key] = newValue
                    }
                )) {
                    ForEach(0 ..< 24, id: \.self) { hour in
                        Text("\(hour)").tag(hour)
                    }
                }
            }

        case "ABPMInt_first", "ABPMInt_second", "CBPInt_first", "CBPInt_second":
            if let measurementTimeValue = settingValue[key] as? Int {
                Picker("", selection: Binding(
                    get: {
                        measurementTimeValue
                    },
                    set: { newValue in
                        settingValue[key] = newValue
                    }
                )) {
                    ForEach(MeasurementTime.allCases) { time in
                        Text(time.description).tag(time.rawValue)
                    }
                }
            }

        case "CBP_zone2_meas_off", "CBP_zone1_meas_off", "SW_SEL_silent", "SW_checkhide", "BP_meas_off", "CBP_meas_off", "SW_AUTO_hide", "SW_AUS_Hide", "SW_AVG_no_include_first", "SW_CBP", "SW_AFib", "SW_AMPM", "SW_Kpa", "AutoMeasureNumber":
            if let boolValue = settingValue[key] as? Bool {
                Toggle("", isOn: Binding(
                    get: {
                        boolValue
                    },
                    set: { newValue in
                        settingValue[key] = newValue
                    }
                ))
            }

        case "AUS_HI_infPressure", "HI_infPressure":
            if let hiInfPressureValue = settingValue[key] as? Int {
                Picker("", selection: Binding(
                    get: {
                        hiInfPressureValue
                    },
                    set: { newValue in
                        settingValue[key] = newValue
                    }
                )) {
                    ForEach(HIinfPressure.allCases) { pressure in
                        Text(pressure.description).tag(pressure.rawValue)
                    }
                }
            }

        case "Interval":
            if let interval = settingValue[key] as? Int {
                Picker("", selection: Binding(
                    get: {
                        interval
                    },
                    set: { newValue in
                        settingValue[key] = newValue
                    }
                )) {
                    ForEach(Interval.allCases) { pressure in
                        Text(pressure.description).tag(pressure.rawValue)
                    }
                }
            }

        case "RestTime", "IntervalTime":
            if let intValue = settingValue[key] as? Int {
                Picker("", selection: Binding(
                    get: {
                        intValue
                    },
                    set: { newValue in
                        settingValue[key] = newValue
                    }
                )) {
                    ForEach(0 ..< 255, id: \.self) { val in
                        Text("\(val)").tag(val)
                    }
                }
            }

        case "No.":
            if let intValue = settingValue[key] as? Int {
                HStack {
                    Text("\(intValue)")
                    Spacer()
                }
            }

        default:
            EmptyView()
        }
    }
}

#Preview {
    ContentView()
}
