//
//  MenuView.swift
//  MicroLifeDeviceSDK-Demo_Swift
//
//  Created by 吳憲有 on 2024/1/18.
//

import SwiftUI
import UIKit

struct MenuView: View {
    var viewModel = MenuViewModel()

    var appInfo = AppInfo.shared

    @State private var backgroundTask: UIBackgroundTaskIdentifier = .invalid

    init() {
        viewModel.setupBLESDK()
    }

    var body: some View {
        NavigationView {
            VStack {
                List(viewModel.menus) { menuInfo in
                    NavigationLink(destination: destinationView(for: menuInfo)) {
                        MenuCell(menuInfo: menuInfo)
                    }
                    .listRowSeparator(.hidden)
                    .background(menuInfo.backgroundColor)
                }
                .listStyle(PlainListStyle())
                .navigationBarTitle("DeviceSDK v"+MicroLifeDeviceSDKVersionNumber.description, displayMode: .automatic)
                .refreshable {
                    viewModel.refreshData()
                }
                .onChange(of: appInfo.showViewType) {
                    if appInfo.showViewType == .menu {
                        viewModel.setupBLESDK()
                    }
                }
                .onAppear {
                    NotificationCenter.default.addObserver(forName: UIApplication.didEnterBackgroundNotification, object: nil, queue: .main) { _ in
                        
                    }
                }

                if viewModel.state != .poweredOn {
                    Button {
                        if let appSettings = URL(string: UIApplication.openSettingsURLString) {
                            UIApplication.shared.open(appSettings, options: [:], completionHandler: nil)
                        }
                    } label: {
                        HStack {
                            Image(systemName: "antenna.radiowaves.left.and.right.slash")
                            Text("Please turn on Bluetooth")
                        }
                    }
                }
                HStack {
                    Spacer() // 占据中间空间
                    if let appVersion = Bundle.main.infoDictionary?["CFBundleShortVersionString"] as? String,
                       let appBVersion = Bundle.main.infoDictionary?["CFBundleVersion"] as? String
                    {
                        Text("V \(appVersion)(\(appBVersion))")
                            .padding(.trailing, 10)
                    }
                }
            }
        }
    }

    private func destinationView(for menuInfo: MenuInfo) -> some View {
        switch menuInfo.deviceType {
            case 102:
                return AnyView(SupportView(menuInfo: menuInfo).environment(appInfo))
            case 103:
                return AnyView(OFAView(menuInfo: menuInfo).environment(appInfo))
            case DeviceType.type3GBP.rawValue:
                return AnyView(BP3gView(menuInfo: menuInfo).environment(appInfo))
            case DeviceType.type4GBP.rawValue:
                return AnyView(BP4gView(menuInfo: menuInfo).environment(appInfo))
            case DeviceType.typeWeight.rawValue:
                return AnyView(WeightView(menuInfo: menuInfo).environment(appInfo))
            case DeviceType.typeTemperature.rawValue:
                return AnyView(TemperatureView(menuInfo: menuInfo).environment(appInfo))
            case DeviceType.typeOxygen.rawValue:
                return AnyView(OxygenView(menuInfo: menuInfo).environment(appInfo))
            case DeviceType.typeWatchBPHome.rawValue:
                return AnyView(WatchBPHomeView(menuInfo: menuInfo).environment(appInfo))
            case DeviceType.typeWatchBPO3.rawValue:
                return AnyView(WatchBPO3View(menuInfo: menuInfo).environment(appInfo))
            case DeviceType.typeWatchBPOffice.rawValue:
                return AnyView(WatchBPOfficeView(menuInfo: menuInfo).environment(appInfo))
            case DeviceType.typeWatchBPM1.rawValue:
                return AnyView(WatchBPM1View(menuInfo: menuInfo).environment(appInfo))
            case DeviceType.typeBGM700.rawValue:
                return AnyView(BGM700View(menuInfo: menuInfo).environment(appInfo))
            case DeviceType.typeBGM800.rawValue:
                return AnyView(BGM800View(menuInfo: menuInfo).environment(appInfo))
            default:
                return AnyView(EmptyView())
        }
    }
}

struct MenuCell: View {
    var menuInfo: MenuInfo

    var body: some View {
        VStack(alignment: .leading) {
            Text(menuInfo.title)
                .font(.largeTitle)
                .multilineTextAlignment(.leading)
                .foregroundColor(menuInfo.textColor)
            if let device = menuInfo.device {
                HStack {
                    Text("Name:"+device.name)
                        .font(.subheadline)
                        .multilineTextAlignment(.leading)
                        .foregroundColor(menuInfo.textColor)
                    Spacer()
                    Text("RSSI:"+device.rssi.stringValue)
                        .font(.subheadline)
                        .multilineTextAlignment(.leading)
                        .foregroundColor(menuInfo.textColor)
                    Spacer()
                }
                Text("UUID:"+device.uuid)
                    .font(.subheadline)
                    .multilineTextAlignment(.leading)
                    .foregroundColor(menuInfo.textColor)
            }
        }
    }
}

#Preview {
    MenuView()
}
