//
//  ToolBox.swift
//  MicroLifeDeviceSDK-Demo_Swift
//
//  Created by 吳憲有 on 2024/1/18.
//

import Combine
import Foundation
import SwiftUI
import UIKit

/// Device Orientation
enum DeviceOrientation {
    /// iPhone Horizontal
    case iPhoneHorizontal
    /// iPhone Vertical
    case iPhoneVertical
    /// iPad
    case iPad
    /// Other
    case OtherDevice
    /// 檢查裝置方向
    static func checkOrientation(horizontalSizeClass: UserInterfaceSizeClass?, verticalSizeClass: UserInterfaceSizeClass?) -> DeviceOrientation {
        if let horizontalSizeClass = horizontalSizeClass, let verticalSizeClass = verticalSizeClass {
            print("horizontalSizeClass: \(horizontalSizeClass), verticalSizeClass: \(verticalSizeClass)")
            if horizontalSizeClass == .regular && verticalSizeClass == .compact {
                return .iPhoneHorizontal
            } else if horizontalSizeClass == .compact && verticalSizeClass == .regular {
                return .iPhoneVertical
            } else if horizontalSizeClass == .regular && verticalSizeClass == .regular {
                return .iPad
            } else {
                return .OtherDevice
            }
        } else {
            return .OtherDevice
        }
    }
}

/// Color Type
enum ColorType: Int {
    /// 暗黑色
    case darkBlack = 0
    /// 暗色
    case dark
    /// 暖色
    case warm
    /// 亮色
    case bright
    /// 原色
    case primary
}

extension Color {
    /// 創建隨機顏色
    /// - Parameters:
    ///   - type: 顏色類型
    ///   - alpha: 透明度
    /// - Returns: 隨機顏色
    static func RainbowColor(type: ColorType) -> Color {
        return RainbowColor(type: type, alpha: .random(in: 0...1))
    }

    /// 創建隨機顏色
    /// - Parameters:
    ///   - type: 顏色類型
    ///   - alpha: 透明度
    /// - Returns: 隨機顏色
    static func RainbowColor(type: ColorType, alpha: Double) -> Color {
        let randomIndex = Double((type == .primary) ? 1000001 : 250001)
        let scaleFactor = 0.000001
        let adjustmentFactor = 0.25
        let typeIndex = Double(type.rawValue % 4)
        let red = adjustColor(randomIndex: randomIndex, scaleFactor: scaleFactor, adjustmentFactor: adjustmentFactor, typeIndex: typeIndex)
        let green = adjustColor(randomIndex: randomIndex, scaleFactor: scaleFactor, adjustmentFactor: adjustmentFactor, typeIndex: typeIndex)
        let blue = adjustColor(randomIndex: randomIndex, scaleFactor: scaleFactor, adjustmentFactor: adjustmentFactor, typeIndex: typeIndex)
        return Color(red: red, green: green, blue: blue, opacity: alpha)
    }

    /// 調整顏色值
    /// - Parameters:
    ///   - randomIndex: 用於生成隨機值的索引範圍
    ///   - scaleFactor: 縮放因子
    ///   - adjustmentFactor: 調整因子
    ///   - typeIndex: 類型索引
    /// - Returns: 調整後的顏色值
    private static func adjustColor(randomIndex: Double, scaleFactor: Double, adjustmentFactor: Double, typeIndex: Double) -> Double {
        var value = Double.random(in: 0...randomIndex) * scaleFactor
        value += adjustmentFactor * typeIndex
        return value
    }

    /// 從十六進制字符串創建顏色
    /// - Parameter hexString: 十六進制字符串表示的顏色
    /// - Returns: 對應的顏色
    static func ColorFromHexString(hexString: String) -> Color {
        var cleanString = hexString.replacingOccurrences(of: "#", with: "")
        if cleanString.count == 3 {
            let index0 = cleanString.index(cleanString.startIndex, offsetBy: 0)
            let index1 = cleanString.index(cleanString.startIndex, offsetBy: 1)
            let index2 = cleanString.index(cleanString.startIndex, offsetBy: 2)
            cleanString = "\(cleanString[index0])\(cleanString[index0])\(cleanString[index1])\(cleanString[index1])\(cleanString[index2])\(cleanString[index2])"
        }
        if cleanString.count == 6 {
            cleanString = "\(cleanString)ff"
        }
        let baseValue = UInt32(cleanString, radix: 16) ?? 0
        let red = Double((baseValue >> 24) & 0xFF) / 255.0
        let green = Double((baseValue >> 16) & 0xFF) / 255.0
        let blue = Double((baseValue >> 8) & 0xFF) / 255.0
        let alpha = Double(baseValue & 0xFF) / 255.0
        return Color(red: red, green: green, blue: blue, opacity: alpha)
    }

    /// 計算互補顏色
    /// - Parameter color: 原始顏色
    /// - Returns: 互補顏色
    static func ComplementaryColor(color: Color) -> Color {
        let components = colorComponents(from: color, complementary: true)
        return Color(red: components.red, green: components.green, blue: components.blue, opacity: components.alpha)
    }

    /// 計算互補顏色
    /// - Parameters:
    ///   - color: 原始顏色
    ///   - alpha: 透明度
    /// - Returns: 互補顏色
    static func ComplementaryColor(color: Color, alpha: Double) -> Color {
        let components = colorComponents(from: color, complementary: true)
        return Color(red: components.red, green: components.green, blue: components.blue, opacity: alpha)
    }

    /// 提取顏色的 RGBA 分量
    /// - Parameters:
    ///   - color: 顏色對象
    ///   - complementary: 是否計算互補色（默認為 false）
    /// - Returns: 顏色的 RGBA 分量
    private static func colorComponents(from color: Color, complementary: Bool = false) -> (red: Double, green: Double, blue: Double, alpha: Double) {
        let uiColor = UIColor(color)
        let components = uiColor.cgColor.components ?? [0.0, 0.0, 0.0, 1.0]
        let red = Double(components[0])
        let green = Double(components[1])
        let blue = Double(components[2])
        let alpha = Double(components[3])
        if complementary {
            let red = 1.0 - red
            let green = 1.0 - green
            let blue = 1.0 - blue
            let alpha = 1.0 - alpha
            let complementaryAlpha = alpha == 0 ? 0.1 : alpha
            return (red, green, blue, complementaryAlpha)
        }
        return (red, green, blue, alpha)
    }

    /// 取得當前畫面背景顏色
    static func currentBackground(alpha: Double) -> Color {
        return Color(UIColor.systemBackground).opacity(alpha)
    }
    
    /// 取得當前畫面文字顏色
    static func currentText(alpha: Double) -> Color {
        return Color(UIColor.label).opacity(alpha)
    }
    
    /// 取得當前畫面分隔線顏色
    static func currentSeparator(alpha: Double) -> Color {
        return Color(UIColor.separator).opacity(alpha)
    }
    
    /// 取得當前畫面選擇顏色
    static func currentSelection(alpha: Double) -> Color {
        return Color(UIColor.systemBlue).opacity(alpha)
    }
    
}

struct KeyboardAwareModifier: ViewModifier {
    @State private var keyboardHeight: CGFloat = 0

    func body(content: Content) -> some View {
        content
            .padding(.bottom, keyboardHeight)
            .edgesIgnoringSafeArea(keyboardHeight > 0 ? .bottom : [])
            .onReceive(Publishers.keyboardHeight) { keyboardHeight in
                withAnimation {
                    self.keyboardHeight = keyboardHeight
                }
            }
    }
}

extension Publishers {
    static var keyboardHeight: AnyPublisher<CGFloat, Never> {
        let willShow = NotificationCenter.default.publisher(for: UIApplication.keyboardWillShowNotification)
            .map { ($0.userInfo?[UIResponder.keyboardFrameEndUserInfoKey] as? CGRect)?.height ?? 0 }
        let willHide = NotificationCenter.default.publisher(for: UIApplication.keyboardWillHideNotification)
            .map { _ in CGFloat(0) }
        return MergeMany(willShow, willHide)
            .eraseToAnyPublisher()
    }
}

/// 鍵盤適應
extension View {
    /// 在键盘弹出时，整个视图都会被上推，以避免键盘遮挡输入框
    func keyboardAwarePadding() -> some View {
        ModifiedContent(content: self, modifier: KeyboardAwareModifier())
    }
    
}
