//
//  WatchBPHomeViewModel.swift
//  MicroLifeDeviceSDK-Demo_Swift
//
//  Created by 吳憲有 on 2024/5/17.
//

import Foundation

@Observable
class WatchBPHomeViewModel {
    
    var deviceConnected = watchBPHome.deviceConnected
    
    func setupBLESDK() {
        
        log.clear()

        sdk.cancelAllConnect()
        
        watchBPHome.autoConnect(true)

        watchBPHome.getDidUpdateStateBlock { state in
            log.T(text: "isBLEStatePoweredOn: \(state)")
        } scanDeviceBlock: { checkMacType, device in
            if let microLifeDevice = device as? MicroLifeDevice {
                log.T(text: "ScanDevice:\nName:\(microLifeDevice.name) MacType:\(checkMacType.description) RSSI:\(microLifeDevice.rssi)\nUUID:\(microLifeDevice.uuid)")
            }
        } cancelScanBlock: {
            log.T(text: "Cancel Scan")
        } connectDeviceStateBlock: { device, _, _, state, error in
            log.T(text: "Device: \(device)\nPeripheral State: \(state)")
            log.E(error: error)
        } cancelAllDevicesConnectionBlock: {
            log.T(text: "Cancel All Devices Connection")
            watchBPHome.startScan()
        }
        
        watchBPHome.deviceConnectedBlock {
            self.deviceConnected = watchBPHome.deviceConnected
        }
        
        watchBPHome.getReadDataModelBlock { CMD, model, error in
            log.M(CMD: CMD, model: model)
            log.E(error: error)
            guard let command = CMDWatchBPHome(rawValue: CMD) else {
                return
            }
            switch command {
            case .readUsualModeHistoryData:
                // Handle reading usual mode history data
                // You may want to process the 'model' data here
                break
            case .readUsualModeHistoryDataIncludeEachMeasurement:
                // Handle reading usual mode history data including each measurement
                break
            case .readDiagnosticModeHistoryData:
                // Handle reading diagnostic mode history data
                break
            case .clearHistoryDataMode:
                // Handle clearing history data mode
                break
            case .clearCurrentModeHistoryData:
                // Handle clearing current mode history data
                break
            case .disconnect:
                // Handle disconnecting
                break
            case .writeDeviceTime:
                // Handle writing device time
                break
            case .writeUserID:
                // Handle writing user ID
                break
            case .readNocturnalModeSetting:
                // Handle reading nocturnal mode setting
                break
            case .changeNocturnalMode:
                // Handle changing nocturnal mode
                break
            case .readDeviceSettingOfMeasurement:
                // Handle reading device setting of measurement
                break
            case .writeDeviceSettingOfMeasurement:
                // Handle writing device setting of measurement
                break
            case .readDeviceInfo:
                // Handle reading device information
                break
            case .readDeviceTime:
                // Handle reading device time
                break
            case .readUserAndVersionData:
                // Handle reading user and version data
                break
            case .readNocturnalModeHistoryData:
                // Handle reading nocturnal mode history data
                break
            case .readSerialNumber:
                // Handle reading device serial number
                break
            default:
                break
            }
        }
        
        watchBPHome.getValidationErrorBlock { item, info, value in
            log.T(text: "\(item)\ninfo:\(info)\nvalue:\(value)")
        }
    }
    
    func refreshData() {
        log.clear()
        log.T(text: "refreshData")
        disconnectDevice()
        startScan()
    }
    
    func startScan() {
        watchBPHome.startScan()
        log.T(text: "Start Scan")
    }

    func cancelScan() {
        watchBPHome.cancelScan()
        log.T(text: "Cancel Scan")
    }
    
    func disconnectDevice() {
        if deviceConnected {
            watchBPHome.disconnect()
            watchBPHome.disconnectDevice()
        }
        log.T(text: deviceConnected ? "unConnect" : "disconnectDevice")
    }
    
    /// WatchBPHome command
    func readUsualModeHistoryData() {
        watchBPHome.readUsualModeHistoryData()
        log.T(text: "Reading Usual Mode History Data")
    }

    func readDiagnosticModeHistoryData() {
        watchBPHome.readDiagnosticModeHistoryData()
        log.T(text: "Reading Diagnostic Mode History Data")
    }

    func clearHistoryData() {
        let usualMode = Bool.random()
        let diagnosticMode = Bool.random()
        let nocturnalMode = Bool.random()
        watchBPHome.clearHistoryData(withSelectedUsualMode: usualMode, diagnosticMode: diagnosticMode, nocturnalMode: nocturnalMode)
        log.T(text: "Selected >> UsualMode: \(usualMode ? "YES" : "NO") DiagnosticMode: \(diagnosticMode ? "YES" : "NO") NocturnalMode: \(nocturnalMode ? "YES" : "NO")")
    }

    func clearCurrentModeHistoryData() {
        watchBPHome.clearCurrentModeHistoryData()
        log.T(text: "Clearing Current Mode History Data")
    }

    func disconnect() {
        watchBPHome.disconnect()
        log.T(text: "Disconnecting")
    }

    func writeDeviceTime() {
        watchBPHome.writeDeviceTime()
        log.T(text: "Writing Device Time")
    }

    func writeUserID() {
        let ID = getNumAndLetterID(length: 11)
        watchBPHome.writeUserID(ID)
        log.T(text: "Writing User ID: \(ID)")
    }

    func readNocturnalModeSetting() {
        watchBPHome.readNocturnalModeSetting()
        log.T(text: "Reading Nocturnal Mode Setting")
    }

    func changeNocturnalModeSetting() {
        let changeNocturnalModeSetting = Bool.random()
        let dateComponents = Calendar.current.dateComponents([.year, .month], from: Date())
        let month = Int.random(in: 1...(dateComponents.month ?? 1))
        let dayMax: Int
        switch month {
        case 2:
            dayMax = 29
        case 4, 6, 9, 11:
            dayMax = 30
        default:
            dayMax = 31
        }
        let day = Int.random(in: 1...dayMax)
        let hours = ["0", "1", "2", "3", "4", "5", "6", "22", "23"]
        let hour = hours.randomElement() ?? "0"
        watchBPHome.changeNocturnalModeSetting(on: changeNocturnalModeSetting, startYear: dateComponents.year ?? 2024, startMonth: month, startDay: day, startHour: Int(hour) ?? 0)
        log.T(text: "changeNocturnalModeSetting: \(changeNocturnalModeSetting ? "YES" : "NO") \(dateComponents.year ?? 2024)/\(month)/\(day) \(hour) \nNote: This command requires matching hardware to set. You can use “readDeviceIDAndInfo(MicroLifeDeviceInfo.openNocturnalMode)” to check if the device supports it.")
    }

    func readDeviceIDAndInfo() {
        watchBPHome.readDeviceIDAndInfo()
        log.T(text: "Reading Device ID and Info")
    }

    func readDeviceTime() {
        watchBPHome.readDeviceTime()
        log.T(text: "Reading Device Time")
    }

    func readUserAndVersionData() {
        watchBPHome.readUserAndVersionData()
        log.T(text: "Reading User and Version Data")
    }

    func readNocturnalModeHistoryData() {
        watchBPHome.readNocturnalModeHistoryData()
        log.T(text: "Reading Nocturnal Mode History Data")
    }

    func readSerialNumber() {
        watchBPHome.readSerialNumber()
        log.T(text: "Reading Serial Number")
    }

    func writeDeviceSettingOfMeasurement() {
        let measurement = Int.random(in: 1...2)
        let restTime = Int.random(in: 0...5)
        let intervalTime = Int.random(in: 0...5)
        let excludeAverage = Int.random(in: 0...1)
        let SWAfib = Bool.random()
        watchBPHome.writeDeviceSetting(ofMeasurement: MeasurementTimes(rawValue: measurement) ?? .time1, restTime: RestIntervalTime(rawValue: restTime) ?? .time60Seconds, intervalTime: RestIntervalTime(rawValue: intervalTime) ?? .time30Seconds, excludeAverage: ExcludeAverage(rawValue: excludeAverage) ?? .AverageAllMeasurements, swAfib: SWAfib)
        log.T(text: "Selected >> measurement: \(measurement) RestTime: \(restTime) IntervalTime: \(intervalTime) ExcludeAverage: \(excludeAverage == 0 ? "average all measurements" : "average excludes 1st measurement") SWAfib: \(SWAfib ? "YES" : "NO")")
    }

    func readDeviceSettingOfMeasurement() {
        watchBPHome.readDeviceSettingOfMeasurement()
        log.T(text: "Reading Device Setting of Measurement")
    }

    func readUsualModeHistoryDataIncludeEachMeasurement() {
        watchBPHome.readUsualModeHistoryDataIncludeEachMeasurement()
        log.T(text: "Reading Usual Mode History Data Including Each Measurement")
    }
    
}
