//
//  WatchBPM1View.swift
//  MicroLifeDeviceSDK-Demo_Swift
//
//  Created by 吳憲有 on 2024/5/31.
//

import SwiftUI

struct WatchBPM1View: View {
    
    @Environment(AppInfo.self) var appInfo
    
    @Environment(\.horizontalSizeClass) var horizontalSizeClass
    
    @Environment(\.verticalSizeClass) var verticalSizeClass
    
    var viewModel = WatchBPM1ViewModel()
    
    var menuInfo: MenuInfo
    
    var body: some View {
        let orientation = DeviceOrientation.checkOrientation(horizontalSizeClass: horizontalSizeClass, verticalSizeClass: verticalSizeClass)
        ZStack {
            switch orientation {
            case .iPhoneHorizontal, .OtherDevice:
                HorizontalLayoutView(scale: 0.6) {
                    viewModel.startScan()
                } cancelScanAction: {
                    viewModel.cancelScan()
                } disconnectDeviceAction: {
                    viewModel.disconnectDevice()
                } commandViewContent: {
                    CustomWatchBPM1View(viewModel: viewModel)
                }
            case .iPhoneVertical, .iPad:
                VerticalLayoutView(scale: 0.75) {
                    viewModel.startScan()
                } cancelScanAction: {
                    viewModel.cancelScan()
                } disconnectDeviceAction: {
                    viewModel.disconnectDevice()
                } commandViewContent: {
                    CustomWatchBPM1View(viewModel: viewModel)
                }
            }
        }
        .toolbar {
            CustomToolbarItem(
                filePath: log.filePath,
                fileName: log.fileName,
                refreshAction: {
                    viewModel.refreshData()
                })
        }
        .navigationTitle(menuInfo.title)
        .padding(.horizontal, 8)
        .onChange(of: appInfo.showViewType) {
            if appInfo.showViewType == .watchBPM1 {
                viewModel.setupBLESDK()
            }
        }
        .onAppear {
            appInfo.showViewType = .watchBPM1
            viewModel.setupBLESDK()
        }
        .onDisappear {
            appInfo.showViewType = .menu
            viewModel.disconnectDevice()
        }
    }
}

struct CustomWatchBPM1View: View {
    var viewModel: WatchBPM1ViewModel
    
    @State private var showSettingValueView = false

    @State private var settings: [String: Any] = [:]

    @State private var showText: String = "Write Setting Values"

    var body: some View {
        VStack(alignment: .trailing, spacing: 16) {
            Button(action: {
                viewModel.readAllHistorys()
            }) {
                Text("Read All History Data")
            }
            Button(action: {
                viewModel.clearAllHistorys()
            }) {
                Text("Clear All History Data")
            }
            Button(action: {
                viewModel.disconnect()
            }) {
                Text("Disconnect")
            }
            Button(action: {
                viewModel.readUserAndVersionData()
            }) {
                Text("Read User and Version Data")
            }
            Button(action: {
                viewModel.writeUserID()
            }) {
                Text("Write User ID")
            }
            Button(action: {
                viewModel.readSettingValues()
            }) {
                Text("Read Setting Values")
            }
            if viewModel.showSettings {
                Button(action: {
                    settings = viewModel.settings.parseDictionary() as! [String: Any]
                    showText = "Write Setting Values"
                    showSettingValueView.toggle()
                }) {
                    Text(showText)
                }
                .sheet(isPresented: $showSettingValueView) {
                    let showLists = ["AUS_HI_infPressure", "HI_infPressure", "SW_AUTO_hide", "SW_SEL_silent", "SW_AUS_Hide", "SW_AVG_no_include_first", "SW_CBP", "SW_AFib", "SW_AMPM", "SW_Kpa", "RestTime", "IntervalTime", "AutoMeasureNumber"]
                    SettingValueView(showLists: showLists, settings: $settings, showText: $showText) {
                        viewModel.settings = MicroLifeWatchBPSettingValues.parseObject(withKeyValues: settings)
                        if viewModel.settings.checkSettingValues() {
                            showSettingValueView = false
                            viewModel.writeSettingValues()
                        } else {
                            showSettingValueView = true
                            showText = "Please set the correct value"
                        }
                    }
                }
            }
            Button(action: {
                viewModel.readDeviceIDAndInfo()
            }) {
                Text("Read Device ID and Information")
            }
            Button(action: {
                viewModel.readDeviceTime()
            }) {
                Text("Read Device Time")
            }
            Button(action: {
                viewModel.writeDeviceTime()
            }) {
                Text("Write Device Time")
            }
            Button(action: {
                viewModel.readFunctionSettingValue()
            }) {
                Text("Read Function Setting Value")
            }
            Button(action: {
                viewModel.readBTModuleName()
            }) {
                Text("Read Bluetooth Module Name")
            }
        }
        .padding()
    }
}


#Preview {
    WatchBPM1View(menuInfo: MenuInfo())
}
