//
//  WatchBPOfficeView.swift
//  MicroLifeDeviceSDK-Demo_Swift
//
//  Created by 吳憲有 on 2024/5/21.
//

import SwiftUI

struct WatchBPOfficeView: View {
    
    @Environment(AppInfo.self) var appInfo
    
    @Environment(\.horizontalSizeClass) var horizontalSizeClass
    
    @Environment(\.verticalSizeClass) var verticalSizeClass
    
    var viewModel = WatchBPOfficeViewModel()
    
    var menuInfo: MenuInfo
    
    var body: some View {
        let orientation = DeviceOrientation.checkOrientation(horizontalSizeClass: horizontalSizeClass, verticalSizeClass: verticalSizeClass)
        ZStack {
            switch orientation {
            case .iPhoneHorizontal, .OtherDevice:
                HorizontalLayoutView(scale: 0.6) {
                    viewModel.startScan()
                } cancelScanAction: {
                    viewModel.cancelScan()
                } disconnectDeviceAction: {
                    viewModel.disconnectDevice()
                } commandViewContent: {
                    CustomWatchBPOfficeView(viewModel: viewModel)
                }
            case .iPhoneVertical, .iPad:
                VerticalLayoutView(scale: 0.75) {
                    viewModel.startScan()
                } cancelScanAction: {
                    viewModel.cancelScan()
                } disconnectDeviceAction: {
                    viewModel.disconnectDevice()
                } commandViewContent: {
                    CustomWatchBPOfficeView(viewModel: viewModel)
                }
            }
        }
        .toolbar {
            CustomToolbarItem(
                filePath: log.filePath,
                fileName: log.fileName,
                refreshAction: {
                    viewModel.refreshData()
                })
        }
        .navigationTitle(menuInfo.title)
        .padding(.horizontal, 8)
        .onChange(of: appInfo.showViewType) {
            if appInfo.showViewType == .watchBPOffice {
                viewModel.setupBLESDK()
            }
        }
        .onAppear {
            appInfo.showViewType = .watchBPOffice
            viewModel.setupBLESDK()
        }
        .onDisappear {
            appInfo.showViewType = .menu
            viewModel.disconnectDevice()
        }
    }
}

struct CustomWatchBPOfficeView: View {
    var viewModel: WatchBPOfficeViewModel
    
    @State var historyMeasuremeCount = 0

    @State private var CBPindex = 0
    
    @State private var dformat = Dformat.fullCBPRaw
    
    @State private var showSettingValueView = false

    @State private var settings: [String: Any] = [:]

    @State private var showText: String = "Write Setting Values"

    var body: some View {
        VStack(alignment: .trailing, spacing: 16) {
            HStack {
                if viewModel.historyMeasuremeCount > 0 {
                    CBPindexPicker(historyMeasuremeCount: viewModel.historyMeasuremeCount, CBPindex: $CBPindex) {
                        viewModel.CBPindex = CBPindex
                    }
                }
                DformatPicker(dformat: $dformat) {
                    viewModel.dformat = dformat
                }
            }
            Button(action: {
                viewModel.readAllHistorys()
            }) {
                Text("Read All History Data")
            }
            if viewModel.historyMeasuremeCount > 0 {
                Button(action: {
                    viewModel.readCBPData()
                }) {
                    Text("Read CBP Data")
                }
            }
            Button(action: {
                viewModel.clearAllHistorys()
            }) {
                Text("Clear All History Data")
            }
            Button(action: {
                viewModel.disconnect()
            }) {
                Text("Disconnect")
            }
            Button(action: {
                viewModel.readUserAndVersionData()
            }) {
                Text("Read User and Version Data")
            }
            Button(action: {
                viewModel.writeUserID()
            }) {
                Text("Write User ID")
            }
            Button(action: {
                viewModel.readSettingValues()
            }) {
                Text("Read Setting Values")
            }
            if viewModel.showSettings {
                Button(action: {
                    settings = viewModel.settings.parseDictionary() as! [String: Any]
                    showText = "Write Setting Values"
                    showSettingValueView.toggle()
                }) {
                    Text(showText)
                }
                .sheet(isPresented: $showSettingValueView) {
                    let showLists = ["AUS_HI_infPressure", "HI_infPressure", "SW_AUTO_hide", "SW_SEL_silent", "SW_AUS_Hide", "SW_AVG_no_include_first", "SW_CBP", "SW_AFib", "SW_AMPM", "SW_Kpa", "RestTime", "IntervalTime", "AutoMeasureNumber"]
                    SettingValueView(showLists: showLists, settings: $settings, showText: $showText) {
                        viewModel.settings = MicroLifeWatchBPSettingValues.parseObject(withKeyValues: settings)
                        if viewModel.settings.checkSettingValues() {
                            showSettingValueView = false
                            viewModel.writeSettingValues()
                        } else {
                            showSettingValueView = true
                            showText = "Please set the correct value"
                        }
                    }
                }
            }
            Button(action: {
                viewModel.readDeviceIDAndInfo()
            }) {
                Text("Read Device ID and Information")
            }
            Button(action: {
                viewModel.readDeviceTime()
            }) {
                Text("Read Device Time")
            }
            Button(action: {
                viewModel.writeDeviceTime()
            }) {
                Text("Write Device Time")
            }
            Button(action: {
                viewModel.readFunctionSettingValue()
            }) {
                Text("Read Function Setting Value")
            }
            Button(action: {
                viewModel.readBTModuleName()
            }) {
                Text("Read Bluetooth Module Name")
            }
            Button(action: {
                viewModel.startRemoteMeasurement()
            }) {
                Text("Start Remote Measurement\n\(viewModel.dformat.description)")
            }
            Button(action: {
                viewModel.stopRemoteMeasurement()
            }) {
                Text("Stop Remote Measurement")
            }
        }
        .padding()
    }
}

#Preview {
    WatchBPOfficeView(menuInfo: MenuInfo())
}
