//
//  WeightViewModel.swift
//  MicroLifeDeviceSDK-Demo_Swift
//
//  Created by 吳憲有 on 2024/5/10.
//

import Foundation
import BelterScaleInfo

@Observable
class WeightViewModel {

    var deviceConnected = false
    
    func setupBLESDK() {
        
        log.clear()

        sdk.cancelAllConnect()
        
        weighingScale.autoConnect(true)

        weighingScale.getDidUpdateStateBlock { state in
            log.T(text:"isBLEStatePoweredOn: \(state)")
        } scanDeviceBlock: { checkMacType, device in
            if let microLifeDevice = device as? MicroLifeDevice {
                log.T(text: "ScanDevice:\nName:\(microLifeDevice.name) MacType:\(checkMacType.description) RSSI:\(microLifeDevice.rssi)\nUUID:\(microLifeDevice.uuid)")
            }
        } cancelScanBlock: {
            log.T(text:"Cancel Scan")
        } connectDeviceStateBlock: { device, central, peripheral, state, error in
            log.T(text:"Device: \(device)\nPeripheral State: \(state)")
            log.E(error: error)
        } cancelAllDevicesConnectionBlock: {
            log.T(text:"Cancel All Devices Connection")
        }
        
        weighingScale.deviceConnectedBlock {
            self.deviceConnected = weighingScale.deviceConnected
            writeUserData()
        }
        weighingScale.getReadDataModelBlock { CMD, model, error in
            log.M(CMD: CMD, model: model)
            log.E(error: error)
            guard let command = CMDWeight(rawValue: CMD) else {
                return
            }
            switch command {
            case .wakeUpScale:
                // Handle waking up the scale
                break
            case .sleepScale:
                // Handle putting the scale to sleep
                break
            case .measurementResult:
                if let bodyFat = model as? MicroLifeBodyFat {
                    self.scaleBleManagerDidReceiveMeasureResult(bodyFat: bodyFat)
                }
                break
            case .readHistorys:
                if let bodyFat = model as? MicroLifeBodyFat {
                    self.scaleBleManagerDidReceiveMeasureResult(bodyFat: bodyFat)
                }
                break
            case .updateUserInfo:
                // Handle updating user information
                break
            case .clearAllUserInfo:
                // Handle clearing all user information
                break
            case .noMoreOfflineData:
                // Handle indication of no more offline data
                break
            case .lowPower:
                // Handle low power indication
                break
            case .syncSystemClock:
                // Handle synchronizing system clock
                break
            default:
                // Handle other cases if needed
                break
            }
        }
        
        weighingScale.getValidationErrorBlock { item, info, value in
            log.T(text: "\(item)\ninfo:\(info)\nvalue:\(value)")
        }
    }
    
    func refreshData() {
        log.clear()
        log.T(text:"refreshData")
        disconnectDevice()
        startScan()
    }
    
    func startScan() {
        weighingScale.startScan()
        log.T(text: "Start Scan")
    }

    func cancelScan() {
        weighingScale.cancelScan()
        log.T(text: "Cancel Scan")
    }
    
    func disconnectDevice() {
        if deviceConnected {
            weighingScale.disconnect()
            weighingScale.disconnectDevice()
        }
        log.T(text: deviceConnected ? "unConnect" : "disconnectDevice")
    }
    
    /// Weighing Scale command
    func clearAllUsers() {
        weighingScale.clearAllUsers()
        log.T(text: "Clearing All Users")
    }

    func readHistorys() {
        weighingScale.readHistorys()
        log.T(text: "Reading History")
    }

    func disconnect() {
        weighingScale.disconnect()
        log.T(text: "Disconnecting")
    }

    
    /// Scale 量測解析功能
    func scaleBleManagerDidReceiveMeasureResult(bodyFat: MicroLifeBodyFat) {
        
        appInfo.user.resistance = bodyFat.resistance
        
        let sex: Int32 = (appInfo.user.gender == .female) ? 0 : 1
        let height = appInfo.user.height.doubleValue
        let weight = bodyFat.weight.doubleValue
        let resistance = bodyFat.resistance.int32Value
        let age = appInfo.user.age.int32Value
        var type: BTRoleType = BTRoleTypeNormal
        if (appInfo.user.roleType != .normal) {
            type = BTRoleTypeSportsman
        }
        
        if let scaleInfo = BelterScaleInfo.getInstance() {
            scaleInfo.setUserInfo_height(height, weight: weight, resistance: resistance, sex: sex, age: age, csRoleType: type)
            bodyFat.bmi = NSNumber(value: scaleInfo.getBMI())
            bodyFat.visceralfat = NSNumber(value: scaleInfo.getVisceralfat())
            bodyFat.bmr = NSNumber(value: scaleInfo.getBMR())
            bodyFat.muscle = NSNumber(value: scaleInfo.getMuscle())
            bodyFat.protein = NSNumber(value: scaleInfo.getProtein())
            bodyFat.bone = NSNumber(value: scaleInfo.getBone())
            bodyFat.moisture = NSNumber(value: scaleInfo.getMoisture())
            bodyFat.boneMuscle = NSNumber(value: scaleInfo.getBoneMuscle())
            bodyFat.bodyScore = NSNumber(value: scaleInfo.getBodyScore())
            bodyFat.bodyAge = NSNumber(value: scaleInfo.getBodyAge())
        }
        
        log.T(text:"BelterScaleInfo:\(bodyFat.parseDictionary())")
    }
    
}


